@@ -7,16 +7,17 @@ module FakeFS
     attr_reader :path
 
     def self._check_for_valid_file(path)
-      raise Errno::ENOENT, path unless FileSystem.find(path)
+      raise Errno::ENOENT, path.to_s unless FileSystem.find(path)
     end
 
     def initialize(string)
       self.class._check_for_valid_file(string)
 
-      @path = FileSystem.normalize_path(string)
-      @open = true
-      @pointer = 0
+      @path     = FileSystem.normalize_path(string)
+      @open     = true
+      @pointer  = 0
       @contents = ['.', '..'] + FileSystem.find(@path).entries
+      @inode    = FakeInode.new(self)
     end
 
     def close
@@ -79,7 +80,7 @@ module FakeFS
 
     def self.delete(string)
       _check_for_valid_file(string)
-      raise Errno::ENOTEMPTY, string unless FileSystem.find(string).empty?
+      raise Errno::ENOTEMPTY, string.to_s unless FileSystem.find(string).empty?
 
       FileSystem.delete(string)
     end
@@ -116,9 +117,9 @@ module FakeFS
       Dir.open(dirname) { |file| yield file }
     end
 
-    def self.glob(pattern, _flags = 0, &block)
+    def self.glob(pattern, flags = 0, &block)
       matches_for_pattern = lambda do |matcher|
-        [FileSystem.find(matcher) || []].flatten.map do |e|
+        [FileSystem.find(matcher, flags, true) || []].flatten.map do |e|
           if Dir.pwd.match(%r{\A/?\z}) ||
              !e.to_s.match(%r{\A#{Dir.pwd}/?})
             e.to_s
@@ -218,6 +219,10 @@ module FakeFS
       end
     end
 
+    def ino
+      @inode.inode_num
+    end
+
     # This code has been borrowed from Rubinius
     def self.mktmpdir(prefix_suffix = nil, tmpdir = nil)
       case prefix_suffix
