@@ -138,43 +138,6 @@ module FakeFS
       self.class.new(@path.chomp(ext) + repl)
     end
 
-    # chop_basename(path) -> [pre-basename, basename] or nil
-    def chop_basename(path)
-      base = File.basename(path)
-      if /\A#{SEPARATOR_PAT}?\z/o =~ base
-        return nil
-      else
-        return path[0, path.rindex(base)], base
-      end
-    end
-    private :chop_basename
-
-    # split_names(path) -> prefix, [name, ...]
-    def split_names(path)
-      names = []
-      while (r = chop_basename(path))
-        path, basename = r
-        names.unshift basename
-      end
-
-      [path, names]
-    end
-
-    private :split_names
-
-    def prepend_prefix(prefix, relpath)
-      if relpath.empty?
-        File.dirname(prefix)
-      elsif /#{SEPARATOR_PAT}/o =~ prefix
-        prefix = File.dirname(prefix)
-        prefix = File.join(prefix, '') if File.basename(prefix + 'a') != 'a'
-        prefix + relpath
-      else
-        prefix + relpath
-      end
-    end
-    private :prepend_prefix
-
     # Returns clean pathname of +self+ with consecutive slashes and
     # useless dots removed. The filesystem is not accessed.
     #
@@ -192,97 +155,6 @@ module FakeFS
       end
     end
 
-    #
-    # Clean the path simply by resolving and removing excess
-    # "." and ".." entries.
-    # Nothing more, nothing less.
-    #
-    def cleanpath_aggressive
-      path = @path
-      names = []
-      pre = path
-      while (r = chop_basename(pre))
-        pre, base = r
-        case base
-        when '.' # rubocop:disable Lint/EmptyWhen
-        when '..'
-          names.unshift base
-        else
-          if names[0] == '..'
-            names.shift
-          else
-            names.unshift base
-          end
-        end
-      end
-      if /#{SEPARATOR_PAT}/o =~ File.basename(pre)
-        names.shift while names[0] == '..'
-      end
-      self.class.new(prepend_prefix(pre, File.join(*names)))
-    end
-    private :cleanpath_aggressive
-
-    # trailing_separator?(path) -> bool
-    def trailing_separator?(path)
-      if (r = chop_basename(path))
-        pre, basename = r
-        pre.length + basename.length < path.length
-      else
-        false
-      end
-    end
-
-    private :trailing_separator?
-
-    # add_trailing_separator(path) -> path
-    def add_trailing_separator(path)
-      if File.basename(path + 'a') == 'a'
-        path
-      else
-        # xxx: Is File.join is appropriate to add separator?
-        File.join(path, '')
-      end
-    end
-    private :add_trailing_separator
-
-    def del_trailing_separator(path)
-      if (r = chop_basename(path))
-        pre, basename = r
-        pre + basename
-      elsif /#{SEPARATOR_PAT}+\z/o =~ path
-        $` + File.dirname(path)[/#{SEPARATOR_PAT}*\z/o]
-      else
-        path
-      end
-    end
-    private :del_trailing_separator
-
-    def cleanpath_conservative
-      path = @path
-      names = []
-      pre = path
-      while (r = chop_basename(pre))
-        pre, base = r
-        names.unshift base if base != '.'
-      end
-      if /#{SEPARATOR_PAT}/o =~ File.basename(pre)
-        names.shift while names[0] == '..'
-      end
-      if names.empty?
-        self.class.new(File.dirname(pre))
-      else
-        names << '.' if names.last != '..' && File.basename(path) == '.'
-
-        result = prepend_prefix(pre, File.join(*names))
-        if /\A(?:\.|\.\.)\z/ !~ names.last && trailing_separator?(path)
-          self.class.new(add_trailing_separator(result))
-        else
-          self.class.new(result)
-        end
-      end
-    end
-    private :cleanpath_conservative
-
     #
     # Returns the real (absolute) pathname of +self+ in the actual
     # filesystem not containing symlinks or useless dots.
@@ -438,54 +310,6 @@ module FakeFS
     end
     alias / +
 
-    def plus(path1, path2) # -> path
-      prefix2 = path2
-      index_list2 = []
-      basename_list2 = []
-      while (r2 = chop_basename(prefix2))
-        prefix2, basename2 = r2
-        index_list2.unshift prefix2.length
-        basename_list2.unshift basename2
-      end
-
-      return path2 if prefix2 != ''
-
-      prefix1 = path1
-      while (r1 = chop_basename(prefix1))
-        while !basename_list2.empty? && basename_list2.first == '.'
-          index_list2.shift
-          basename_list2.shift
-        end
-
-        prefix1, basename1 = r1
-        next if basename1 == '.'
-        if basename1 == '..' ||
-           basename_list2.empty? ||
-           basename_list2.first != '..'
-          prefix1 += basename1
-          break
-        end
-        index_list2.shift
-        basename_list2.shift
-      end
-
-      r1 = chop_basename(prefix1)
-      if !r1 && /#{SEPARATOR_PAT}/o =~ File.basename(prefix1)
-        while !basename_list2.empty? && basename_list2.first == '..'
-          index_list2.shift
-          basename_list2.shift
-        end
-      end
-
-      if !basename_list2.empty?
-        suffix2 = path2[index_list2.first..-1]
-        r1 ? File.join(prefix1, suffix2) : prefix1 + suffix2
-      else
-        r1 ? prefix1 : File.dirname(prefix1)
-      end
-    end
-    private :plus
-
     #
     # Pathname#join joins pathnames.
     #
@@ -625,6 +449,173 @@ module FakeFS
         Pathname.new(File.join(*relpath_names))
       end
     end
+
+    private
+
+    # chop_basename(path) -> [pre-basename, basename] or nil
+    def chop_basename(path)
+      base = File.basename(path)
+      if /\A#{SEPARATOR_PAT}?\z/o =~ base
+        return nil
+      else
+        return path[0, path.rindex(base)], base
+      end
+    end
+
+    # split_names(path) -> prefix, [name, ...]
+    def split_names(path)
+      names = []
+      while (r = chop_basename(path))
+        path, basename = r
+        names.unshift basename
+      end
+
+      [path, names]
+    end
+
+    def prepend_prefix(prefix, relpath)
+      if relpath.empty?
+        File.dirname(prefix)
+      elsif /#{SEPARATOR_PAT}/o =~ prefix
+        prefix = File.dirname(prefix)
+        prefix = File.join(prefix, '') if File.basename(prefix + 'a') != 'a'
+        prefix + relpath
+      else
+        prefix + relpath
+      end
+    end
+
+    #
+    # Clean the path simply by resolving and removing excess
+    # "." and ".." entries.
+    # Nothing more, nothing less.
+    #
+    def cleanpath_aggressive
+      path = @path
+      names = []
+      pre = path
+      while (r = chop_basename(pre))
+        pre, base = r
+        case base
+        when '.' # rubocop:disable Lint/EmptyWhen
+        when '..'
+          names.unshift base
+        else
+          if names[0] == '..'
+            names.shift
+          else
+            names.unshift base
+          end
+        end
+      end
+      if /#{SEPARATOR_PAT}/o =~ File.basename(pre)
+        names.shift while names[0] == '..'
+      end
+      self.class.new(prepend_prefix(pre, File.join(*names)))
+    end
+
+    # trailing_separator?(path) -> bool
+    def trailing_separator?(path)
+      if (r = chop_basename(path))
+        pre, basename = r
+        pre.length + basename.length < path.length
+      else
+        false
+      end
+    end
+
+    # add_trailing_separator(path) -> path
+    def add_trailing_separator(path)
+      if File.basename(path + 'a') == 'a'
+        path
+      else
+        # xxx: Is File.join is appropriate to add separator?
+        File.join(path, '')
+      end
+    end
+
+    def del_trailing_separator(path)
+      if (r = chop_basename(path))
+        pre, basename = r
+        pre + basename
+      elsif /#{SEPARATOR_PAT}+\z/o =~ path
+        $` + File.dirname(path)[/#{SEPARATOR_PAT}*\z/o]
+      else
+        path
+      end
+    end
+
+    def cleanpath_conservative
+      path = @path
+      names = []
+      pre = path
+      while (r = chop_basename(pre))
+        pre, base = r
+        names.unshift base if base != '.'
+      end
+      if /#{SEPARATOR_PAT}/o =~ File.basename(pre)
+        names.shift while names[0] == '..'
+      end
+      if names.empty?
+        self.class.new(File.dirname(pre))
+      else
+        names << '.' if names.last != '..' && File.basename(path) == '.'
+
+        result = prepend_prefix(pre, File.join(*names))
+        if /\A(?:\.|\.\.)\z/ !~ names.last && trailing_separator?(path)
+          self.class.new(add_trailing_separator(result))
+        else
+          self.class.new(result)
+        end
+      end
+    end
+
+    def plus(path1, path2) # -> path
+      prefix2 = path2
+      index_list2 = []
+      basename_list2 = []
+      while (r2 = chop_basename(prefix2))
+        prefix2, basename2 = r2
+        index_list2.unshift prefix2.length
+        basename_list2.unshift basename2
+      end
+
+      return path2 if prefix2 != ''
+
+      prefix1 = path1
+      while (r1 = chop_basename(prefix1))
+        while !basename_list2.empty? && basename_list2.first == '.'
+          index_list2.shift
+          basename_list2.shift
+        end
+
+        prefix1, basename1 = r1
+        next if basename1 == '.'
+        if basename1 == '..' ||
+           basename_list2.empty? ||
+           basename_list2.first != '..'
+          prefix1 += basename1
+          break
+        end
+        index_list2.shift
+        basename_list2.shift
+      end
+
+      r1 = chop_basename(prefix1)
+      if !r1 && /#{SEPARATOR_PAT}/o =~ File.basename(prefix1)
+        while !basename_list2.empty? && basename_list2.first == '..'
+          index_list2.shift
+          basename_list2.shift
+        end
+      end
+
+      if !basename_list2.empty?
+        suffix2 = path2[index_list2.first..-1]
+        r1 ? File.join(prefix1, suffix2) : prefix1 + suffix2
+      else
+        r1 ? prefix1 : File.dirname(prefix1)
+      end
+    end
   end
 
   # Pathname class
@@ -796,9 +787,12 @@ module FakeFS
     end
 
     # See <tt>File.write</tt>. Returns the number of bytes written.
-    def write(path, data)
-      File.open(path, 'wb') do |file|
-        return file.write(data)
+    def write(string, *args)
+      offset = args[0]
+      open_args = args[1]
+      File.open(@path, open_args || 'w') do |file|
+        file.seek(offset) if offset
+        return file.write(string)
       end
     end
   end
@@ -885,9 +879,9 @@ module FakeFS
       FileTest.setgid?(@path)
     end
 
-    # See <tt>FileTest.size</tt>.
+    # See <tt>FileTest.size?</tt>.
     def size
-      FileTest.size(@path)
+      FileTest.size?(@path)
     end
 
     # See <tt>FileTest.size?</tt>.
